import cStringIO
import traceback

from soaplib.soap import make_soap_envelope, make_soap_fault, from_soap, collapse_swa, apply_mtom
from soaplib.service import SoapServiceBase
from soaplib.util import reconstruct_url
from soaplib.serializers.primitive import string_encoding, Fault
from soaplib.etimport import ElementTree
from threading import local

request = local()

_exceptions = False
_exception_logger = None

_debug = False
_debug_logger = None

###################################################################
# Logging / Debugging Utilities                                   #
# replace with python logger?                                     #
###################################################################

def _dump(e): # util?
    print e

def log_exceptions(on, out=_dump):
    global _exceptions
    global _exception_logger

    _exceptions=on
    _exception_logger=out

def log_debug(on,out=_dump):
    global _debug
    global _debug_logger
    
    _debug=on
    _debug_logger=out
    
def debug(msg):
    global _debug
    global _debug_logger
    
    if _debug:
        _debug_logger(msg)
        
def exceptions(msg):
    global _exceptions
    global _exception_logger
    
    if _exceptions:
        _exception_logger(msg)

def reset_request():
    '''
    This method clears the data stored in the threadlocal 
    request object
    '''
    request.environ = None
    request.header = None
    request.additional = {}

class WSGISoapApp(object):
    '''
    This is the base object representing a soap web application, and conforms
    to the WSGI specification (PEP 333).  This object should be overridden
    and getHandler(environ) overridden to provide the object implementing
    the specified functionality.  Hooks have been added so that the subclass
    can react to various events that happen durring the execution of the
    request.
    '''
    
    def onCall(self,environ):
        '''
        This is the first method called when this WSGI app is invoked
        @param the wsgi environment
        '''
        pass
    
    def onWsdl(self,environ,wsdl):
        '''
        This is called when a wsdl is requested
        @param the wsgi environment
        @param the wsdl string 
        '''
        pass
    
    def onWsdlException(self,environ,exc,resp):
        '''
        Called when an exception occurs durring wsdl generation
        @param the wsgi environment
        @param exc the exception
        @param the fault response string
        '''
        pass
    
    def onMethodExec(self,environ,body,py_params,soap_params):
        '''
        Called BEFORE the service implementing the functionality is called
        @param the wsgi environment
        @param the body element of the soap request
        @param the tuple of python params being passed to the method
        @param the soap elements for each params
        '''
        pass
    
    def onResults(self,environ,py_results,soap_results):
        '''
        Called AFTER the service implementing the functionality is called
        @param the wsgi environment
        @param the python results from the method
        @param the xml serialized results of the method
        '''
        pass
    
    def onException(self,environ,exc,resp):
        '''
        Called when an error occurs durring execution
        @param the wsgi environment
        @param the exception
        @param the response string
        '''
        pass
    
    def onReturn(self,environ,returnString):
        '''
        Called before the application returns
        @param the wsgi environment
        @param return string of the soap request
        '''
        pass
    
    def getHandler(self,environ):
        '''
        This method returns the object responsible for processing a given request, and
        needs to be overridden by a subclass to handle the application specific
        mapping of the request to the appropriate handler.
        @param the wsgi environment
        @returns the object to be called for the soap operation
        '''
        raise Exception("Not implemented")
    
    def __call__(self, environ, start_response, address_url=None):
        '''
        This method conforms to the WSGI spec for callable wsgi applications (PEP 333).
        This method looks in environ['wsgi.input'] for a fully formed soap request envelope,
        will deserialize the request parameters and call the method on the object returned
        by the getHandler() method.
        @param the http environment
        @param a callable that begins the response message
        @returns the string representation of the soap call
        '''
        methodname = ''
        try:
            reset_request()
            request.environ = environ
            
            # implementation hook
            self.onCall(environ)
            
            serviceName = environ['PATH_INFO'].split('/')[-1]
            service = self.getHandler(environ)
            if (environ['QUERY_STRING'].endswith('wsdl') or environ['PATH_INFO'].endswith('wsdl')) and environ['REQUEST_METHOD'].lower() == 'get':
                # get the wsdl for the service
                #
                # Assume path_info matches pattern
                # /stuff/stuff/stuff/serviceName.wsdl or ?WSDL
                #
                serviceName = serviceName.split('.')[0]
                if address_url:
                    url = address_url
                else:
                    url = reconstruct_url(environ).split('.wsdl')[0]
                
                start_response('200 OK',[('Content-type','text/xml')])
                try:
                    wsdl_content = service.wsdl(url)
                    
                    # implementation hook
                    self.onWsdl(environ,wsdl_content)
                except Exception, e:
                    
                    # implementation hook
                    buffer = cStringIO.StringIO()
                    traceback.print_exc(file=buffer)
                    buffer.seek(0)
                    stacktrace = str(buffer.read())
                    faultStr = ElementTree.tostring(make_soap_fault( str(e), detail=stacktrace), encoding=string_encoding)
                    
                    exceptions(faultStr)
                    
                    self.onWsdlException(environ,e,faultStr)
                    
                    # initiate the response
                    start_response('500',[('Content-type','text/xml'),('Content-length',str(len(faultStr)))])
                    return [faultStr]
                
                reset_request()
                return [wsdl_content]
            
            if environ['REQUEST_METHOD'].lower() != 'post':
                start_response('405 Method Not Allowed',[('Allow','POST')])
                return ''
                
            input = environ.get('wsgi.input')
            length = environ.get("CONTENT_LENGTH")
            body = input.read(int(length))
            debug(body)
            body, _unmentioned_attachs = collapse_swa( environ.get("CONTENT_TYPE"), body)
            
            # deserialize the body of the message
            try:
                payload, header = from_soap(body)
            except SyntaxError,e:
                payload = None
                header = None

            if payload:
                methodname = payload.tag.split('}')[-1]
            else:
                # check HTTP_SOAPACTION
                methodname = environ.get("HTTP_SOAPACTION")
                if methodname.startswith('"') and methodname.endswith('"'):
                    methodname = methodname[1:-1]
                if methodname.find('/') >0:
                    methodname = methodname.split('/')[1]

            request.header = header

            # call the method
            func = getattr(service, methodname)
            
            # retrieve the method descriptor
            descriptor = func(_soap_descriptor=True, klazz=service.__class__)
            if payload:
                params = descriptor.inMessage.from_xml(*[payload])
            else:
                params = ()
            # implementation hook
            self.onMethodExec(environ,body,params,descriptor.inMessage.params)
            
            # call the method
            if len(_unmentioned_attachs) > 0:
            	retval = func(*params, unmentioned_attachs = _unmentioned_attachs)
            else:
            	retval = func(*params)
            
            # transform the results into an element
            # only expect a single element
            results = None
            if not (descriptor.isAsync or descriptor.isCallback):
                results = descriptor.outMessage.to_xml(*[retval])

            # implementation hook
            self.onResults(environ,results,retval)
            
            # grab any headers that were included in the request
            response_headers = None
            if hasattr(request,'response_headers'):
                response_headers = request.response_headers
            
            # construct the soap response, and serialize it
            envelope = make_soap_envelope(results,tns=service.__tns__,header_elements=response_headers) 
            ElementTree.cleanup_namespaces(envelope)
            resp = ElementTree.tostring(envelope, encoding=string_encoding)
            headers = {'Content-Type': 'text/xml'}

            if descriptor.mtom:
                headers, resp = apply_mtom( headers, resp,
                                            descriptor.outMessage.params,
                                            (retval,) )

            
            if environ.has_key('CONTENT_LENGTH'):
                del(environ['CONTENT_LENGTH'])
                
            # initiate the response
            start_response('200 OK',headers.items())
            
            self.onReturn(environ,resp)
            
            debug(resp)
            
            # return the serialized results
            reset_request()
            return [resp]
        
        except Fault,e:

	        # grab any headers that were included in the request
            response_headers = None
            if hasattr(request,'response_headers'):
                response_headers = request.response_headers
	    
            # The user issued a Fault, so handle it just like an exception!
            fault = make_soap_fault(
                e.faultstring,
                e.faultcode,
                e.detail,
                header_elements = response_headers)
                
            faultStr = ElementTree.tostring(fault, encoding=string_encoding)
            exceptions(faultStr)
            
            self.onException(environ,e,faultStr)
            reset_request()

            # initiate the response
            start_response('500 Internal Server Error',[('Content-type','text/xml')])
            return [faultStr]
        
        except Exception, e:
            # Dump the stack trace to a buffer to be sent
            # back to the caller
            
            # capture stacktrace
            buffer = cStringIO.StringIO()
            traceback.print_exc(file=buffer)
            buffer.seek(0)
            stacktrace = str(buffer.read())
            
            faultstring = str(e)
            if methodname:
                faultcode = faultCode='%sFault'%methodname
            else:
                faultcode = 'Server'
            detail = stacktrace
            
            faultStr = ElementTree.tostring(make_soap_fault(faultstring,faultcode,detail), encoding=string_encoding)            
            exceptions(faultStr)
            
            self.onException(environ,e,faultStr)
            reset_request()
            
            # initiate the response
            start_response('500 Internal Server Error',[('Content-type','text/xml')])
            return [faultStr]

class SimpleWSGISoapApp(WSGISoapApp, SoapServiceBase):
    '''
    This object is a VERY simple extention of the base WSGISoapApp.
    It subclasses both WSGISoapApp, and SoapServiceBase, so that
    an object can simply subclass this single object, and it will
    be both a wsgi application and a soap service.  This is convenient
    if you want to only expose some functionality, and dont need
    complex handler mapping, and all of the functionality can be put
    in a single class.
    '''
    def __init__(self):
        WSGISoapApp.__init__(self)
        SoapServiceBase.__init__(self)
    
    def getHandler(self,environ):
        return self
        
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>
import os
import re
import sys
import tempfile
from os import path
from time import time
from random import random

try:
    from hashlib import sha1
except ImportError:
    from sha import new as sha1
try:
    from cPickle import dumps, loads, HIGHEST_PROTOCOL
except ImportError:
    from pickle import dumps, loads, HIGHEST_PROTOCOL

_sha1_re = re.compile(r'^[a-f0-9]{40}$')

def _urandom():
    if hasattr(os, 'urandom'):
        return os.urandom(30)
    return random()

def generate_key(salt=None):
    return sha1('%s%s%s' % (salt, time(), _urandom())).hexdigest()

class UpdateDictMixin(object):
    """Makes dicts call `self.on_update` on modifications.

    .. versionadded:: 0.5

    :private:
    """

    on_update = None

    def calls_update(name):
        def oncall(self, *args, **kw):
            rv = getattr(super(UpdateDictMixin, self), name)(*args, **kw)
            if self.on_update is not None:
                self.on_update(self)
            return rv
        oncall.__name__ = name
        return oncall

    __setitem__ = calls_update('__setitem__')
    __delitem__ = calls_update('__delitem__')
    clear = calls_update('clear')
    pop = calls_update('pop')
    popitem = calls_update('popitem')
    setdefault = calls_update('setdefault')
    update = calls_update('update')
    del calls_update

class CallbackDict(UpdateDictMixin, dict):
    """A dict that calls a function passed every time something is changed.
    The function is passed the dict instance.
    """

    def __init__(self, initial=None, on_update=None):
        dict.__init__(self, initial or ())
        self.on_update = on_update

    def __repr__(self):
        return '<%s %s>' % (
            self.__class__.__name__,
            dict.__repr__(self)
        )

class ModificationTrackingDict(CallbackDict):
    __slots__ = ('modified',)

    def __init__(self, *args, **kwargs):
        def on_update(self):
            self.modified = True
        self.modified = False
        CallbackDict.__init__(self, on_update=on_update)
        dict.update(self, *args, **kwargs)

    def copy(self):
        """Create a flat copy of the dict."""
        missing = object()
        result = object.__new__(self.__class__)
        for name in self.__slots__:
            val = getattr(self, name, missing)
            if val is not missing:
                setattr(result, name, val)
        return result

    def __copy__(self):
        return self.copy()

class Session(ModificationTrackingDict):
    """Subclass of a dict that keeps track of direct object changes.  Changes
    in mutable structures are not tracked, for those you have to set
    `modified` to `True` by hand.
    """
    __slots__ = ModificationTrackingDict.__slots__ + ('sid', 'new')

    def __init__(self, data, sid, new=False):
        ModificationTrackingDict.__init__(self, data)
        self.sid = sid
        self.new = new

    def __repr__(self):
        return '<%s %s%s>' % (
            self.__class__.__name__,
            dict.__repr__(self),
            self.should_save and '*' or ''
        )

    @property
    def should_save(self):
        """True if the session should be saved.

        .. versionchanged:: 0.6
           By default the session is now only saved if the session is
           modified, not if it is new like it was before.
        """
        return self.modified


class SessionStore(object):
    """Baseclass for all session stores.  The Werkzeug contrib module does not
    implement any useful stores besides the filesystem store, application
    developers are encouraged to create their own stores.

    :param session_class: The session class to use.  Defaults to
                          :class:`Session`.
    """

    def __init__(self, session_class=None):
        if session_class is None:
            session_class = Session
        self.session_class = session_class

    def is_valid_key(self, key):
        """Check if a key has the correct format."""
        return _sha1_re.match(key) is not None

    def generate_key(self, salt=None):
        """Simple function that generates a new session key."""
        return generate_key(salt)

    def new(self):
        """Generate a new session."""
        return self.session_class({}, self.generate_key(), True)

    def save(self, session):
        """Save a session."""

    def save_if_modified(self, session):
        """Save if a session class wants an update."""
        if session.should_save:
            self.save(session)

    def delete(self, session):
        """Delete a session."""

    def get(self, sid):
        """Get a session for this sid or a new session object.  This method
        has to check if the session key is valid and create a new session if
        that wasn't the case.
        """
        return self.session_class({}, sid, True)


class MemorySessionStore(SessionStore):
    def __init__(self, session_class=Session):
        SessionStore.__init__(self, session_class)
        self.db = {}

    def save(self, session):
        self.db[session.sid] = session

    def delete(self, session):
        self.db.pop(session.sid, None)

    def get(self, sid):
        try:
            return self.db[sid]
        except KeyError:
            return self.new()


class MemcachedSessionStore(SessionStore):
    def __init__(self, servers, session_class=Session):
        SessionStore.__init__(self, session_class)
        try:
            import cmemcache as memcache
        except ImportError:
            import memcache
        self.client = memcache.Client(servers)

    def save(self, session):
        s = dumps(dict(session), HIGHEST_PROTOCOL)
        self.client.set(str(session.sid), s)

    def delete(self, session):
        self.client.delete(str(session.sid))

    def get(self, sid):
        data = self.client.get(str(sid))
        if data is None:
            return self.session_class({}, sid, False)
        else:
            return self.session_class(loads(data), sid, False)

#class RedisSessionStore(SessionStore):
#    def __init__(self, server="127.0.0.1", port=6379, timeout=30, session_class=Session):
#        SessionStore.__init__(self, session_class)
#        import redis
#        redis_pool = redis.ConnectionPool(host=server,
#                        port=port, db=0,socket_timeout=timeout)
#        self.client = redis.Redis(redis_pool)
#
#    def save(self, session):
#        s = dump(dict(session), HIGHEST_PROTOCOL)
#        self.client.set(str(session.sid), s)
#        self.client.expire(self.session.sid, 300)
#
#    def delete(self, session):
#        self.client.delete(str(session.sid))
#
#    def get(self, sid):
#        from code import interact
#        interact(local=locals())
#        data = self.client.get(str(sid))
#        if data is None:
#            return self.session_class({}, sid, False)
#        else:
#            return self.session_class(loads(data), sid, False)

import cPickle as pickle
from uuid import uuid4
from code import interact
import redis
class RedisPool(object):
    
    _pool = None

    def __init__(self, host, port, timeout=0):
        self.host = host
        self.port = port
        self.timeout = timeout
    def get_pool(self):
        if not RedisPool._pool:
            RedisPool.pool = redis.ConnectionPool(host=self.host,
                port=self.port, db=0,socket_timeout=self.timeout)
        return RedisPool.pool

    def get_redis(self):
        return redis.Redis(connection_pool=self.get_pool())

    
    def _ensure__pool(self):
        if not RedisPool._pool:
            RedisPool.get_pool()
    
    def get_conn(self):
        self._ensure__pool()
        return redis.Redis(connection_pool=self._pool)
    
    def get_pconn(self):
        self._ensure__pool()
        return redis.Redis(connection_pool=self._pool).pipeline()
        # 可以在一次请求中执行多个set命令，这样避免了多次的往返时延

    def clear_rd(self):
        """清空redis数据"""
        rd = self.get_conn()
        r1 = str(rd.dbsize())
        rd.flushdb()
        r2 = str(rd.dbsize())
        #Config.logger.info("已清空redis数据，清空前：%s，清空后：%s" % (r1, r2,))
        print ("已清空redis数据，清空前：%s，清空后：%s" % (r1, r2,))
        
    def clear_rd_keys(self, *patterns):
        """清空redis数据"""
        rd = self.get_conn()
        r1 = str(rd.dbsize())
        for pt in patterns:
            keys = rd.keys(pt)
            for key in keys:
                rd.delete(key)
        r2 = str(rd.dbsize())
        #Config.logger.info("已清空redis %s相关数据，清空前：%s，清空后：%s" % (patterns, r1, r2,))
        print ("已清空redis %s相关数据，清空前：%s，清空后：%s" % (patterns, r1, r2,))

#def get_redis():
#    return RedisPool.get_redis()

class RedisSessionStoreNew:
 
    def __init__(self, redis_pool, **options):
        self.options = {
            'key_prefix': 'RedisSessionStoreNew',
            'expire': 7200,
        }
        self.options.update(options)
        self.redis = redis_pool
 
    def prefixed(self, sid):
        print '%s:%s' % (self.options['key_prefix'], sid)
        return '%s:%s' % (self.options['key_prefix'], sid)
 
    def generate_sid(self, salt=None):
        return generate_key(salt)
#        return uuid4().get_hex()
 
    def get_session(self, sid, name):
        data = self.redis.hget(self.prefixed(sid), name)
        session = pickle.loads(data) if data else dict()
        print "RedisSessionStore--get_session--session", session
        return session
 
    def set_session(self, sid, session_data, name):
        expiry = self.options['expire']
        self.redis.hset(self.prefixed(sid), name, pickle.dumps(session_data))
        if expiry:
            self.redis.expire(self.prefixed(sid), expiry)
        print "save session: set_session--%s-%s-%s" % (sid, session_data, name)
    def save(self, session):
        session._save()
        print "save"
 
    def delete_session(self, sid):
        self.redis.delete(self.prefixed(sid))


class RedisSessionStore:
 
    def __init__(self, redis_connection, **options):
        self.options = {
            'key_prefix': 'session',
            'expire': 7200,
        }
        self.options.update(options)
        self.redis = redis_connection

    def prefixed(self, sid):
        return '%s:%s' % (self.options['key_prefix'], sid)

    def generate_sid(self, salt=None):
        return generate_key(salt)
#        return uuid4().get_hex()

    def get_session(self, sid, name):
        data = self.redis.hget(self.prefixed(sid), name)
        session = pickle.loads(data) if data else dict()
        return session

    def set_session(self, sid, session_data, name):
        expiry = self.options['expire']
        self.redis.hset(self.prefixed(sid), name, pickle.dumps(session_data))
        if expiry:
            self.redis.expire(self.prefixed(sid), expiry)
        #print "save session: set_session--%s-%s-%s" % (sid, session_data, name)

    def save(self, session):
        session._save()

    def delete_session(self, sid):
        self.redis.delete(self.prefixed(sid))


class RedisSession(Session):
    def __init__(self, session_store, sessionid=None):
        self._store = session_store
        self._sessionid = sessionid if sessionid else self._store.generate_sid()
        self._sessiondata = self._store.get_session(self._sessionid, 'data')
        super(RedisSession, self).__init__(self._sessiondata, self._sessionid)
        self.dirty = False
 
    def clear(self):
        self._store.delete_session(self._sessionid)
 
    def access(self, remote_ip):
        access_info = {'remote_ip':remote_ip, 'time':'%.6f' % time()}
        self._store.set_session(
                self._sessionid,
                'last_access',
                pickle.dumps(access_info)
                )
 
    def last_access(self):
        access_info = self._store.get_session(self._sessionid, 'last_access')
        return pickle.loads(access_info)
 
    @property
    def sessionid(self):
        return self._sessionid
 
    def __getitem__(self, key):
        return self._sessiondata[key]
 
    def __setitem__(self, key, value):
        self._sessiondata[key] = value
        self._dirty()
 
    def __delitem__(self, key):
        del self._sessiondata[key]
        self._dirty()
 
    def __len__(self):
        return len(self._sessiondata)
 
    def __contains__(self, key):
        return key in self._sessiondata
 
    def __iter__(self):
        for key in self._sessiondata:
            yield key
 
    def __repr__(self):
        return self._sessiondata.__repr__()
 
    def __del__(self):
        if self.dirty:
            self._save()
 
    def _dirty(self):
        self.dirty = True
        print "dirty:", self.dirty
 
    def _save(self):
        self._store.set_session(self._sessionid, self._sessiondata, 'data')
        self.dirty = False
    @property
    def should_save(self):
        """True if the session should be saved.

        .. versionchanged:: 0.6
           By default the session is now only saved if the session is
           modified, not if it is new like it was before.
        """
        return self.dirty

    def get(self, key, default_value=None):
        try:
            return self[key]
        except KeyError:
            return default_value
        # return self._sessiondata[key]


#try:
#    import pymongo
#
#    class MongoDBSession(BaseSession):
#        """Class implementing the MongoDB based session storage.
#        All sessions are stored in a collection "tornado_sessions" in the db
#        you specify in the session_storage setting.
#
#        The session document structure is following:
#        'session_id': session ID
#        'data': serialized session object
#        'expires': a timestamp of when the session expires, in sec since epoch
#        'user_agent': self-explanatory
#        An index on session_id is created automatically, on application's init.
#
#        The end_request() is called after every operation (save, load, delete),
#        to return the connection back to the pool.
#        """
#
#        def __init__(self, db, **kwargs):
#            super(MongoDBSession, self).__init__(**kwargs)
#            self.db = db # an instance of pymongo.collection.Collection
#            if not kwargs.has_key('session_id'):
#                self.save()
#
#        @staticmethod
#        def _parse_connection_details(details):
#            # mongodb://[host[:port]]/db
#            if details[10] != '/':
#                # host and port specified
#                match = re.match('mongodb://([\S|\.]+?)?(?::(\d+))?/(\S+)', details)
#                host = match.group(1)
#                port = int(match.group(2))
#                database = match.group(3)
#            else:
#                # default host and port
#                host = 'localhost'
#                port = 27017
#                match = re.match('mongodb:///(\S+)', details)
#                database = match.group(1)
#
#            return host, port, database
#
#        def save(self):
#            """Upsert a document to the tornado_sessions collection.
#            The document's structure is like so:
#            {'session_id': self.session_id,
#             'data': self.serialize(),
#             'expires': self._serialize_expires(),
#             'user_agent': self.user_agent}
#            """
#            # upsert
#            self.db.update(
#                {'session_id': self.session_id}, # equality criteria
#                {'session_id': self.session_id,
#                 'data': self.serialize(),
#                 'expires': self._serialize_expires(),
#                 'user_agent': self.user_agent}, # new document
#                upsert=True)
#            self.db.database.connection.end_request()
#
#        @staticmethod
#        def load(session_id, db):
#            """Load session from the storage."""
#            try:
#                data = db.find_one({'session_id': session_id})
#                if data:
#                    kwargs = MongoDBSession.deserialize(data['data'])
#                    db.database.connection.end_request()
#                    return MongoDBSession(db, **kwargs)
#                db.database.connection.end_request()
#                return None
#            except:
#                db.database.connection.end_request()
#                return None
#
#        def delete(self):
#            """Remove session from the storage."""
#            self.db.remove({'session_id': self.session_id})
#            self.db.database.connection.end_request()
#
#        @staticmethod
#        def delete_expired(db):
#            db.remove({'expires': {'$lte': int(time.time())}})
#
#except ImportError:
#    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

def clone_table(table, name, metadata=None):
    from sqlalchemy import Table
    if not metadata:
        metadata = table.metadata
    try:
        return metadata.tables[name]
    except KeyError:
        args = []
        kwargs = {}
        args.append(name)
        args.append(metadata)
        for c in table.columns:
            args.append(c.copy())
        for c in table.constraints:
            args.append(c.copy())
        return Table(*args)


class QuerySet(object):
    def __init__(self, result):
        self.result = result

    def __getslice__(self, x, y):
        return self.result.limit(y-x).offset(x).execute().fetchall()

    def count(self):
        from sqlalchemy import func, select
        return select([func.count()]).select_from(self.result.alias()).execute().scalar()

    def __len__(self):
        return self.count()


class Result(object):
    def __init__(self, lst):
        self.lst = lst

    def get_one(self):
        if not self.lst:
            return None
        return self.lst[0]

    @staticmethod
    def set_variable(variable, value):
        variable.set(value, from_db=True)

    def __iter__(self):
        for row in self.lst:
            yield row

try:
    from storm.locals import create_database
    from storm.locals import Store
    from storm.store import ResultSet
    from storm import Undef
    from storm.exceptions import StormError, NotOneError
    class CacheResultSet(ResultSet):
        def fetchAll(self, cache, key, timeout=0):
            values_list = cache.get(key)
            if values_list is None:
                result = self._store._connection.execute(self._get_select())
                values_list = list(result)
                cache.set(key, values_list, timeout)
            else:
                result = Result(values_list)
            return [self._load_objects(result, values) for values in values_list]
        def fetchOne(self, cache, key, timeout=0):
            values = cache.get(key)
            if values is None:
                select = self._get_select()
                if select.limit is not Undef and select.limit > 2:
                    select.limit = 2
                result = self._store._connection.execute(select)
                values = result.get_one()
                if result.get_one():
                    raise NotOneError("one() used with more than one result available")
                cache.set(key, values, timeout)
            else:
                result = Result([values])
            if values:
                return self._load_objects(result, values)
            return None
    class CacheStore(Store):
        _result_set_factory = CacheResultSet
except ImportError:
    pass

try:
    from sqlalchemy.orm.query import Query
    class CacheQuery(Query):
        def fetchAll(self, cache, key, timeout=0):
            values_list = cache.get(key)
            if values_list is None:
                result_list = self.all()
                cache.set(key, result_list, timeout)
                return result_list
            return values_list
        def fetchOne(self, cache, key, timeout=0):
            value = cache.get(key)
            if value is None:
                value = self.one()
                cache.set(key, value, timeout)
                return value
            return value
        def fetchFirst(self, cache, key, timeout=0):
            value = get(key)
            if value is None:
                value = self.first()
                cache.set(key, value, timeout)
                return value
            return value
except ImportError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

__version__ = "0.1.10"

def get_version():
    return __version__

import os
import re
from os.path import abspath, dirname, join
base_path = abspath(dirname(__file__))
import sys
sys.path.insert(0, abspath(join(base_path, 'utils')))
sys.path.insert(0, abspath(join(base_path, 'lib')))

#from torweb.application import make_application



# -*- coding: utf-8 -*-

from torweb import get_version
#from setuptools import setup, find_packages
from distutils.core import setup

with open('README.md') as file:
    #long_description = file.read()
    readlines = file.readlines()
    long_description = "".join(readlines)

print(long_description)
###
#install_requires=open('requirements.txt').readlines(),
###
setup(
    name = "torweb-for-3",
    version = get_version(),
    packages = ["torweb",
                "torweb.lib",
                "torweb.lib.soaplib",
                "torweb.lib.soaplib.ext",
                "torweb.lib.soaplib.serializers",
                "torweb.lib.soaplib.util",
                "torweb.utils"],
    package_dir = {"torweb":"torweb"},
    package_data = {"torweb":["../*.txt", "../*.md"]},
    include_package_data = True,
    author = "zhangpeng",
    author_email = "zhangpeng1@infohold.com.cn",
    url = "https://github.com/finalbattle/torweb.git",
    description = "torweb",
    long_description=long_description,
    #install_requires=open('requirements.txt').readlines(),
    #install_requires=[
    #    "tornado", "PyYAML", "jinja2", "mysql-python", "storm", "sqlalchemy", "pymemcache"
    #],
)




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

class odict(object):
    """
    Sort of an ordered dictionary implementation.
    """

    class Empty(object):
        pass

    def __init__(self, data=[]):
        if isinstance(data, self.__class__):
            self.__list = list(data.__list)
            self.__dict = dict(data.__dict)

        else:
            self.__list = []
            self.__dict = {}

            self.update(data)

    def __getitem__(self, key):
        if isinstance(key, int):
            return self.__dict[self.__list[key]]
        else:
            return self.__dict[key]

    def __setitem__(self, key, val):
        if isinstance(key, int):
            self.__dict[self.__list[key]] = val

        else:
            if not (key in self.__dict):
                self.__list.append(key)
            self.__dict[key] = val

        assert len(self.__list) == len(self.__dict), (repr(self.__list),
                                                              repr(self.__dict))

    def __contains__(self, what):
        return (what in self.__dict)

    def __repr__(self):
        return "{%s}" % ','.join(["%r: %r" % (k,v) for k,v in self.items()])

    def __str__(self):
        return repr(self)

    def __len__(self):
        assert len(self.__list) == len(self.__dict)

        return len(self.__list)

    def __iter__(self):
        return iter(self.__list)

    def __delitem__(self, key):
        if not isinstance(key, int):
            key = self.__list.index(key) # ouch.

        del self.__dict[self.__list[key]]
        del self.__list[key]

    def items(self):
        for k in self.__list:
            yield k, self.__dict[k]

    def keys(self):
        return self.__list

    def update(self, data):
        if isinstance(data, dict):
            data = data.items()

        for k,v in data:
            self[k] = v

    def values(self):
        for l in self.__list:
            yield self.__dict[l]

    def get(self, key, default=Empty):
        if key in self.__dict:
            return self[key]

        else:
            if default is odict.Empty:
                raise KeyError(key)
            else:
                return default

    def append(self, t):
        k, v = t
        self[k] = v



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from os.path import abspath, dirname

############################################
MYSQL_ROOT_USER = "root"
MYSQL_ROOT_PASS = "1"
############################################

#start_path = abspath(dirname(__file__))
start_path = os.getcwd()
print "start_path:%s" % start_path


init_text = '''#!/usr/bin/python
# -*- coding: utf-8 -*-
# created by start_app

from os.path import abspath, dirname, join
base_path = abspath(dirname(__file__))

# 添加系统路径
import sys
sys.path.insert(0, abspath(join(base_path, '..', 'lib')))

from torweb.config import Yaml_Config
Yaml_Config(base_path, join(base_path, 'settings.yaml'))

metadata = sqlalchemy_config["%(app)s"]["metadata"]
session = sqlalchemy_config["%(app)s"]["session"]

'''

handler_text = '''#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created by start_app

from torweb.handlers import BaseHandler
from %(app)s.shortcuts import *
from code import interact

class Handler(BaseHandler):
    def on_finish(self):
        from %(app)s import session
        session.remove()
        #print 'session removed'
    def render(self, template, **kwargs):
        tmpl = env.get_template(template)
        self.write(tmpl.render(**kwargs))
'''

shortcuts_text = '''#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created by start_app

from torweb.urls import url
from torweb.handlers import *
from code import interact

from %(app)s import *

#cache.clear()

try:
    from storm.locals import *
    from storm.expr import Sum,LeftJoin,Eq,Or,And,Func
    from storm.tracer import debug as storm_debug
except:
    pass

from sqlalchemy import Table
from sqlalchemy.orm import mapper
from sqlalchemy import select, and_, or_
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()

# 设置模板目录
from torweb.tmpl import FragmentCacheExtension
debug = True
from jinja2 import Environment, PackageLoader, MemcachedBytecodeCache, FileSystemBytecodeCache
env = Environment(
    loader = PackageLoader('%(app)s', 'templates'),
    auto_reload = debug,
    extensions = [FragmentCacheExtension],
    bytecode_cache = MemcachedBytecodeCache(cache, prefix="%(app)s/jinja2/bytecode/", timeout=3600*24*8)
)
env.fragment_cache = cache
'''

settings_text = '''app: %(app)s
language: python

static_dir: static
static_url: /static/
static_domain: /static
#static_domain: http://static.%(app)s.com

schemes:
  - scheme: mysql
    alias: %(app)s
    user: %(MYSQL_ROOT_USER)s
    pass: %(MYSQL_ROOT_PASS)s
    host: 127.0.0.1
    port: 3306
    db: %(app)s
    charset: utf8
    storm: store
    sqlalchemy: session

memcached: ['127.0.0.1:11211']

############################################
# SessionStore
# MemcachedSessionStore
############################################
session_store: MemcachedSessionStore
'''

test_models_text = '''#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created by start_app
from %(app)s.shortcuts import *
from torweb.db import CacheQuery

# sqlalchemy
test_model = Table('test_model', metadata, schema='%(app)s', autoload=True)
class Test_Model_Base(Base):
    __table__ = test_model
    __mapper_args__ = {
        'column_prefix':'_'
    }
    @classmethod
    def query(cls):
        return CacheQuery(cls)
    def get_name(self):
        return self._name
    def get_email(self):
        return self._email
    def __repr__(self):
        return u'<test_model '+self._name+'>'
    def __unicode__(self):
        return self.__repr__()

# storm
class Test_Model(Test_Model_Base):
    __storm_table__ = 'test_model'
    id = Int(primary=True)
    name = Unicode(default=u'')
    email = Unicode(default=u'')
'''

test_views_text = '''#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created by start_app

from tornado.web import addslash
from torweb.urls import url
from torweb.handlers import BaseHandler
from %(app)s.handlers import Handler
from code import interact

@url(r'/')
class Index(Handler):
    def get(self):
        self.render('index.html')


@url(r'/test/')
class MyIndex(BaseHandler):
    def get(self, *args, **kwargs):
        self.write('application is running ...')
'''

dev_server_text = '''# -*- coding: utf-8 -*-

import %(app)s 
import tornado.web
import tornado.options
from torweb import run_torweb
from torweb import make_application 
from optparse import OptionParser
from code import interact


if __name__ == '__main__':
    debug = True
    usage = "usage: prog [options] arg1"
    default_port = 8888
    options = OptionParser(usage)
    options.add_option("-p", "--port", dest="port", default=default_port,
                       help="server listenning port, default is 8888",
                       metavar="PORT")
    (option, args) = options.parse_args()
    if debug:
        tornado.options.parse_command_line(args)
    app = make_application(%(app)s, debug, wsgi=False)
    run_torweb.run(app, port=option.port)
'''

command_text = '''# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

import %(app)s
from torweb.command import Command
from code import interact


if __name__ == '__main__':
    command = Command(%(app)s)
    command.run()
'''

test_sql_text = '''--
-- Table structure for table `user_info`
--

DROP TABLE IF EXISTS `user_info`;
/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
CREATE TABLE `user_info` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(255) DEFAULT '',
  `email` varchar(255) DEFAULT '',
  PRIMARY KEY (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;
/*!40101 SET character_set_client = @saved_cs_client */;

--
-- Dumping data for table `user_info`
--

LOCK TABLES `user_info` WRITE;
/*!40000 ALTER TABLE `user_info` DISABLE KEYS */;
/*!40000 ALTER TABLE `user_info` ENABLE KEYS */;
UNLOCK TABLES;
'''

def start_app(app):
    app_path = os.path.join(start_path, app)
    app_init_file = os.path.join(start_path, app, '__init__.py')
    app_handlers_file = os.path.join(start_path, app, 'handlers.py')
    app_shortcuts_file = os.path.join(start_path, app, 'shortcuts.py')
    app_settings_file = os.path.join(start_path, app, 'settings.yaml')
    app_models_path = os.path.join(start_path, app, 'models')
    app_models_init_file = os.path.join(start_path, app, 'models', '__init__.py')
    app_models_test_file = os.path.join(start_path, app, 'models', 'test.py')
    app_views_path = os.path.join(start_path, app, 'views')
    app_views_init_file = os.path.join(start_path, app, 'views', '__init__.py')
    app_views_test_file = os.path.join(start_path, app, 'views', 'test.py')
    app_test_sql = os.path.join(start_path, app, 'models', 'test.sql')
    dev_server_file = os.path.join(start_path, "dev_server.py")
    command_file = os.path.join(start_path, "command.py")
    if not os.path.exists(app_path):os.makedirs(app_path)
    if not os.path.exists(app_init_file):
        with open(app_init_file, 'w') as file:
            file.write(init_text % {'app':app})
            file.close()
    if not os.path.exists(app_handlers_file):
        with open(app_handlers_file, 'w') as file:
            file.write(handler_text % {'app':app})
            file.close()
    if not os.path.exists(app_shortcuts_file):
        with open(app_shortcuts_file, 'w') as file:
            file.write(shortcuts_text % {'app':app})
            file.close()
    if not os.path.exists(app_settings_file):
        with open(app_settings_file, 'w') as file:
            file.write(settings_text % {'app':app, 'MYSQL_ROOT_USER':MYSQL_ROOT_USER, 'MYSQL_ROOT_PASS':MYSQL_ROOT_PASS})
            file.close()
    if not os.path.exists(app_models_path):os.makedirs(app_models_path)
    if not os.path.exists(app_models_init_file):
        with open(app_models_init_file, 'w') as file:
            file.write('')
            file.close()
    if not os.path.exists(app_models_test_file):
        with open(app_models_test_file, 'w') as file:
            file.write(test_models_text % {'app':app, 's':'s'})
            file.close()
    if not os.path.exists(app_views_path):os.makedirs(app_views_path)
    if not os.path.exists(app_views_init_file):
        with open(app_views_init_file, 'w') as file:
            file.write('')
            file.close()
    if not os.path.exists(app_views_test_file):
        with open(app_views_test_file, 'w') as file:
            file.write(test_views_text % {'app':app})
            file.close()
    if not os.path.exists(app_test_sql):
        with open(app_test_sql, 'w') as file:
            file.write(test_sql_text)
            file.close()
    if not os.path.exists(dev_server_file):
        with open(dev_server_file, 'w') as file:
            file.write(dev_server_text % {"app": app})
            file.close()
    if not os.path.exists(command_file):
        with open(command_file, 'w') as file:
            file.write(command_text % {"app": app})
            file.close()

import os
def start_db(app):
    cmd = "mysqladmin -u %(MYSQL_ROOT_USER)s -p%(MYSQL_ROOT_PASS)s create %(app)s" % {'app':app, 'MYSQL_ROOT_USER':MYSQL_ROOT_USER, 'MYSQL_ROOT_PASS':MYSQL_ROOT_PASS}
    os.system(cmd)
    load_cmd = "mysql -u %(MYSQL_ROOT_USER)s -p%(MYSQL_ROOT_PASS)s %(app)s < %(app)s/models/test.sql" % {'app':app, 'MYSQL_ROOT_USER':MYSQL_ROOT_USER, 'MYSQL_ROOT_PASS':MYSQL_ROOT_PASS}
    os.system(load_cmd)
if __name__ == '__main__':
    try:
        if sys.argv[1]:
            start_app(sys.argv[1])
            start_db(sys.argv[1])
            print sys.argv[1]
    except IndexError:
        print u'请输入app名字'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

import os
import yaml
import logging
from code import interact

class Yaml_Config(object):
    def __init__(self, base_path, yaml_path, url_root="/"):
        self.yaml_path = yaml_path
        f = open(self.yaml_path)
        self.CONF = CONFIG(self.yaml_path)
        self.settings = yaml.load(f)
        app_name = self.settings.get('app')
        self.app = __import__(app_name)
        self.app.base_path = base_path
        self.base_path = base_path
        self.init_store()
        self.init_propertys()
        self.set_url_root(url_root)
    def init_propertys(self):
        #self.app.base_path = self.base_path
        self.app.static_path = self.static_path
        self.app.static_url = self.static_url
        self.app.static_domain = self.static_domain
        self.app.cache = self.cache
        self.app.session_store = self.session_store
        self.app.settings = self
    #@property
    #def base_path(self):
    #    if not hasattr(self, '_base_path'):
    #        app_path = os.path.abspath(os.path.dirname(self.app.__file__))
    #        setattr(self, '_base_path', app_path)
    #    return getattr(self, '_base_path')
    @property
    def static_path(self):
        if not hasattr(self, '_static_path'):
            root = self.settings.get('static_root', 'static')
            path = os.path.join(self.base_path, root)
            setattr(self, '_static_path', path)
        return getattr(self, '_static_path')
    @property
    def static_url(self):
        if not hasattr(self, '_static_url'):
            url = self.settings.get('static_url', '/static/')
            setattr(self, '_static_url', url)
        return getattr(self, '_static_url')
    @property
    def static_domain(self):
        if not hasattr(self, '_static_domain'):
            domain = self.settings.get('static_domain', '/static')
            setattr(self, '_static_domain', domain)
        return getattr(self, '_static_domain')
    def set_url_root(self, url_root):
        #url_root = self.settings.get('url_root', '/')
        self.app.url_root = self.url_root = url_root
        from torweb.urls import Url
        self.app.url = Url(url_root)
    @property
    def cache(self):
        if not hasattr(self, '_cache'):
            from torweb.cache import MemcachedCache, NullCache
            from werkzeug.contrib.cache import RedisCache
            
            nullcache = NullCache()
            setattr(self, '_cache', nullcache)
            cache_servers = self.settings.get('memcached', None)
            if cache_servers:
                cache = MemcachedCache(cache_servers)
                setattr(self, '_cache', cache)
            try:
                redis_cache_host = self.CONF("REDIS.HOST")
                redis_cache_port = self.CONF("REDIS.PORT")
                if redis_cache_host and redis_cache_port:
                    cache = RedisCache(redis_cache_host, redis_cache_port)
                    setattr(self, '_cache', cache)
                    logging.info("cache: RedisCache")
            except KeyError:
                pass
            #cache_servers = self.settings.get('memcached', None)
            #if cache_servers:
            #    cache = MemcachedCache(cache_servers)
            #    setattr(self, '_cache', cache)
            #else:
            #    nullcache = NullCache()
            #    setattr(self, '_cache', nullcache)
            pass
        return getattr(self, '_cache')
    @property
    def session_store(self):
        sessionstore_name = self.settings.get('session_store', 'MemorySessionStore')
        module = __import__('torweb.sessions', {}, {}, sessionstore_name) 
        cls =  getattr(module, sessionstore_name)
        #import logging
        #logging.info('session store: %s' % cls.__name__)
        if cls.__name__ == 'MemcachedSessionStore':
            #print 'session store: %s' % cls.__name__
            return cls(self.settings.get('memcached', ['127.0.0.1:11211']))
        if cls.__name__ == 'RedisSessionStore':
            _redis = self.settings.get("redis", {})
            import redis
            redis_conn = redis.StrictRedis(host=_redis.get("host", "127.0.0.1"), port=_redis.get("port", 6379), db="test")
            return cls(redis_conn, key_prefix=_redis.get("prefix", "session"), expire=_redis.get("expire", 3600))
        if cls.__name__ == 'RedisSessionStoreNew':
            _redis = self.settings.get("redis", {})
            import redis
            from torweb.sessions import RedisPool
            redis_pool = RedisPool(_redis.get("host", "127.0.0.1"), _redis.get("port", 6379), _redis.get("expire", 3600))
            return cls(redis_pool.get_redis(), key_prefix=_redis.get("prefix", cls.__name__), expire=_redis.get("expire", 3600))
        if cls.__name__ == 'MemorySessionStore':
            #print 'session store: %s' % cls.__name__
            return cls()
        #print 'session store: default'
        return cls()
    
    def init_store(self, schemes_list=[]):
        schemes = self.settings.get('schemes', schemes_list)
        _scheme_dict = {}
        for scheme in schemes:
            #if not hasattr(self.app, scheme.get('storm')):
            key = scheme.get('scheme', '')
            if key not in ['mysql', 'postgre', 'sqlite']: continue
            connection = '%(scheme)s://%(user)s:%(pass)s@%(host)s:%(port)s/%(database_name)s?charset=%(charset)s'\
                    % {'scheme': key,
                       'user': scheme.get('user', ''),
                       'pass': scheme.get('pass', ''),
                       'host': scheme.get('host', ''),
                       'port': scheme.get('port', ''),
                       'database_name': scheme.get('db', ''),
                       'charset': scheme.get('charset', 'utf8')
                      }
            if key == 'sqlite':
                connection = '%(scheme)s:%(database_name)s'\
                    % {'scheme': key,
                       'database_name': os.path.join(self.base_path, scheme.get('db', ''))
                      }
            store = scheme.get("storm", None)
            if store: self.init_storm(store, connection)
            session = scheme.get("sqlalchemy", None)
            if session: dic = self.init_sqlalchemy(scheme, connection); _scheme_dict["%s" % scheme.get("alias", "") or scheme.get("db", "")] = dic
        setattr(self.app, 'sqlalchemy_config', _scheme_dict)
    def init_storm(self, store, connection):
        try:
            import storm
            from storm.locals import create_database
            from torweb.db import CacheStore
            database = create_database(connection)
            cache_store = CacheStore(database)
            setattr(self, store, cache_store)
            setattr(self.app, store, cache_store)
        except Exception as e:
            print e
    def init_sqlalchemy(self, scheme, connection):
        try:
            import sqlalchemy
            from sqlalchemy import create_engine, MetaData
            from sqlalchemy.orm import scoped_session, sessionmaker
            from torweb.db import CacheQuery
            import _mysql_exceptions
            from sqlalchemy import event
            from sqlalchemy.exc import DisconnectionError
            def my_on_checkout(dbapi_conn, connection_rec, connection_proxy):
                try:
                    dbapi_conn.cursor().execute('select now()')
                except _mysql_exceptions.OperationalError:
                    raise DisconnectionError
            
            engine = create_engine(
                connection,
                convert_unicode=True,
                encoding="utf-8",
                pool_recycle=3600*7,
                #echo_pool=True,
                echo=False,
            )
            event.listen(engine, 'checkout', my_on_checkout)
            metadata = MetaData(bind=engine)
            session = scoped_session(sessionmaker(bind=engine, query_cls=CacheQuery,expire_on_commit=False))
            sqlalchemy_sessions = [session]
            DB_Session = sessionmaker(bind=engine)
            return {"metadata":metadata, "session":session, "sqlalchemy_sessions":sqlalchemy_sessions}
            #setattr(self.app, 'metadata', metadata)
            #setattr(self.app, scheme.get('sqlalchemy', 'session'), session)
            #setattr(self.app, 'sqlalchemy_sessions', sqlalchemy_sessions)
        except Exception as e:
            print e

import re
import logging

inner_pattern = re.compile('\{\w+[\.\w+]*\}')

class ConfigLoader(object):
    config_path = None
    base_path = ""

    def __init__(self):
        self.config = None

    def load_environment(self):
        self.config['project_path'] = self.base_path

    def get_config_path(self):
        if not ConfigLoader.config_path:
            ConfigLoader.config_path = os.path.join(self.base_path, 'config')
        return ConfigLoader.config_path

    def get_config_file(self, name):
        import os.path

        #return os.path.join(self.get_config_path(), '%s.yaml' % name)
        return os.path.join(self.get_config_path(), '%s' % name)

    def load(self, config_file):
        self.config = self.load_file(config_file)
        self.load_environment()
        self.load_dynamic(self.config)
        return self.config
    def load_path(self, config_path):
        self.config = self.load_file(config_path)
        self.load_environment()
        self.load_dynamic(self.config)
        return self.config

    def load_file(self, config_file):
        config = {}
        try:
            with open(self.get_config_file(config_file), 'rb') as f:
                config = yaml.load(f)
                self.load_import(config)
        except Exception:
            logging.error("Uncaught Exception in load_file", exc_info=True)
        return config

    def load_dynamic(self, config):
        for key, value in config.iteritems():
            if isinstance(value, dict):
                self.load_dynamic(value)
                continue

            if not isinstance(value, basestring):
                continue

            matches = inner_pattern.findall(value)
            if not matches:
                continue
            while matches:
                for match in matches:
                    config[key] = re.sub(match, str(self.getConfig(match[1:-1])), config[key])
                matches = inner_pattern.findall(config[key])

    def load_import(self, config):
        if not 'import' in config:
            return config
        import_files = config['import']
        if import_files and (isinstance(import_files, set) or isinstance(import_files, list)):
            for import_file in import_files:
                self.import_single(config, import_file)
        return config

    def combile(self, config, import_config):
        for key, value in import_config.iteritems():
            if key == 'import' and 'import' in config:
                config['import'].extend(value)
                continue

            if not key in config:
                config[key] = value
            elif isinstance(value, dict) and isinstance(config[key], dict):
                self.combile(config[key], value)

    def import_single(self, config, import_file):
        import_config = self.load_file(import_file)
        self.combile(config, import_config)
        return config

    def getConfig(self, key):
        # every key is upper case
        key_no_case = key.lower()
        # split the string by .
        subkeys = re.split(r'\.', key_no_case)
        current_config = self.config
        for subkey in subkeys:
            assert (current_config is not None and isinstance(current_config, dict))
            current_config = current_config[subkey]
        return current_config

def get_host_ip(eth="eth0"):
    ip_addr = None
    import platform

    try:
        if platform.system() == 'Windows':
            import socket
            ip_addr = socket.gethostbyname(socket.getfqdn(socket.gethostname()))
        # 暂时将mac等同于linxu
        else:
            import fcntl
            import struct
            import socket
            def get_ip_address(ifname):
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                return socket.inet_ntoa(fcntl.ioctl(
                    s.fileno(),
                    0x8915, # SIOCGIFADDR
                    struct.pack('256s', ifname[:15])
                )[20:24])

            ip_addr = get_ip_address(eth)
    except Exception as e:
        logging.error(e)

    return ip_addr


class Configuration(object):
    config_context = None

    def __init__(self, config_path):
        if not self.config_context:
            #self.config_context = ConfigLoader().load(config_entry_file)
            self.config_context = ConfigLoader().load_path(config_path)

    def getConfig(self, key):
        #Configuration.init()
        # every key is upper case
        key_no_case = key.lower()
        # split the string by .
        subkeys = re.split(r'\.', key_no_case)
        current_config = self.config_context
        for subkey in subkeys:
            assert (current_config is not None and isinstance(current_config, dict))
            current_config = current_config[subkey]
        return current_config


#configuration = Configuration()
#def CONF(key):
#    """
#     get the configuration value
#    """
#    return configuration.getConfig(key)
#    #return Configuration.getConfig(key)

class CONFIG(object):
    configuration = None
    def __init__(self, path):
        self.configuration = Configuration(path)
    def __call__(self, key):
        return self.configuration.getConfig(key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

#AUTHOR: yeshengzou # # gmail.com
#DATE: 2012.4.23
#LICENCE: GPLv3
  
import PIL
from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont
from random import randint
from cStringIO import StringIO
from code import interact

#CHAR = 'acdefghijkmnpqrstuvwxyABCDEFGHJKLMNPQRSTUVWXY345789'
#LEN = len(CHAR) - 1
#PADDING = 20
#X_SPACE = 10 #两个字符之间最少相隔多少个像素
#TRY_COUNT = 30 #随机字符的位置尝试最多多少次,避免死循环
#WIDTH = 72
#HEIGHT = 26
#LINE_NUM = 2
##FONT = ImageFont.load('font.pil')
##FONT = ImageFont.load('/home/zhangpeng/share/fonts/arial.ttf')
##FONT = ImageFont.truetype('/home/zhangpeng/share/fonts/arial.ttf', 18)
#FONT = ImageFont.truetype('/home/zhangpeng/share/fonts/simsun.ttc', 30)

class Captcha(object):
    def __init__(self, CHAR='acdefghijkmnpqrstuvwxyABCDEFGHJKLMNPQRSTUVWXY345789',
                 PADDING=20, X_SPACE=10, TRY_COUNT=30, WIDTH=72, HEIGHT=26, LINE_NUM=2,
                 FONT='arial.ttf', FONT_SIZE=30,
                 BG_RGBA=(233, 248, 255), FL_RGBA=(48, 133, 18)):
        self.CHAR = CHAR
        self.LEN = len(CHAR) - 1
        self.PADDING = PADDING
        self.X_SPACE = X_SPACE
        self.TRY_COUNT = TRY_COUNT
        self.WIDTH = WIDTH
        self.HEIGHT = HEIGHT
        self.LINE_NUM = LINE_NUM
        self.BG_RGBA = BG_RGBA
        self.FL_RGBA = FL_RGBA
        self.IMAGE_FONT = ImageFont.truetype(FONT, FONT_SIZE)
    def gen(self):
        CHAR = self.CHAR
        LEN = self.LEN
        PADDING = self.PADDING
        X_SPACE = self.X_SPACE
        TRY_COUNT = self.TRY_COUNT
        WIDTH = self.WIDTH
        HEIGHT = self.HEIGHT
        LINE_NUM = self.LINE_NUM
        FONT = self.IMAGE_FONT
        BG_RGBA = self.BG_RGBA
        FL_RGBA = self.FL_RGBA

        #im = Image.new('1', (WIDTH, HEIGHT), 'white')
        im = Image.new('RGBA', (WIDTH, HEIGHT), BG_RGBA)
        draw = ImageDraw.Draw(im)
        w, h = im.size
    
        #S = [(x, y, 'c')]
        S = []
        x_list = []
        y_list = []
        n = 0
        while True:
            n += 1
            if n > TRY_COUNT:
                break
            x = randint(0, w - PADDING)
            flag = True
            for i in x_list:
                if abs(x - i) < X_SPACE:
                    flag = False
                    continue
                if not flag:
                    break
            if not flag:
                continue
    
            y = randint(0, h - PADDING)
            x_list.append(x)
            y_list.append(y)
            S.append((x, y, CHAR[randint(0, LEN)]))
            if len(S) == 4:
                break
    
        for x, y, c in S:
            draw.text((x, y), c, fill=FL_RGBA, font=FONT)
    
        #加3根干扰线
        for i in range(LINE_NUM):
            x1 = randint(0, (w - PADDING) / 2)
            y1 = randint(0, (h- PADDING / 2))
            x2 = randint(0, w)
            y2 = randint((h - PADDING / 2), h)
            draw.line(((x1, y1), (x2, y2)), fill=FL_RGBA, width=1)
    
        S.sort(lambda x, y: 1 if x[0] > y[0] else -1)
        char = [x[2] for x in S]
        fileio = StringIO()
        im.save(fileio, 'gif')
        #im.save('img.gif', 'gif')
        im.show()
        return ''.join(char), fileio

if __name__ == '__main__':
    captcha = Captcha(FONT='/home/zhangpeng/share/fonts/simsun.ttc')
    print captcha.gen()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng1@infohold.com.cn>
"""
# yaml settings:

############################################
# LOGGER
############################################
version: 1
formatters:
  simpleFormater:
    #format: '%(asctime)s - %(levelname)s: %(message)s'
    format: '[%(name)s %(levelname)1.1s %(asctime)s %(module)s:%(lineno)d] %(message)s'
    datefmt: '%Y-%m-%d %H:%M:%S'
  consoleFormatter:
    (): 'echo.logkit.ColoredConsoleFormatter'
  date_formatter:
    format: '[%(levelname)1.1s %(asctime)s %(process)s %(threadName)s  %(module)s:%(lineno)d] %(message)s'
    datefmt: '%Y-%m-%d %H:%M:%S'

handlers:
  console:
    class: logging.StreamHandler
    formatter: consoleFormatter
    level: NOTSET
    stream: ext://sys.stdout
  file:
    class : torweb.logkit.iTimedRotatingFileHandler
    formatter: simpleFormater
    level: NOTSET
    filename: /usr/api-root/logs/echo-static/echo-static/log/info.log
    when: D
  iError:
    class : torweb.logkit.iTimedRotatingFileHandler
    formatter: date_formatter
    level: ERROR
    filename: /usr/api-root/logs/echo-static/echo-static/log/error.log
    when: D

loggers:
  logger:
    level: DEBUG
    #handlers: [console]
    handlers: 
      - console
  tornado:
    level: NOTSET
    qualname: tornado
    handlers: 
      - file
  iError:
    level: ERROR
    qualname: iError
    handlers: 
      - iError

root:
  level: INFO
  handlers: 
    - console
    - iError
    - file

myapp/__init__.py

import yaml
from logging.config import dictConfig

dictConfig(yaml.load(open(settings.yaml_path, 'r')))
itornado = logging.getLogger("console")
logger = logging.getLogger("file")
iError= logging.getLogger("iError")

"""

import time
import logging
from tornado.log import LogFormatter
from tornado.escape import _unicode
from tornado.util import basestring_type

class iFormatter(logging.Formatter):
    def format(self, record):
        if not hasattr(record, 'trace'):
            record.trace = '-'
        return super(iFormatter, self).format(record)

class iTimedRotatingFileHandler(logging.handlers.TimedRotatingFileHandler):
    def shouldRollover(self, record):
        now = time.time()
        t = int(now)
        if t >= self.rolloverAt:
            return 1
        elif (now / self.interval) > (self.rolloverAt / self.interval):
            return 1
        # print "No need to rollover: %d, %d" % (t, self.rolloverAt)
        return 0

class iColoredConsoleFormatter(LogFormatter):
    colored_format = '[%(levelname)1.1s %(asctime)s] %(message)s'
    def format(self, record):
        if not hasattr(record, 'trace'):
            record.trace = '-'
        try:
            record.message = record.getMessage()
        except Exception as e:
            record.message = "Bad message (%r): %r" % (e, record.__dict__)
        assert isinstance(record.message, basestring_type)  # guaranteed by logging
        record.asctime = time.strftime(
            "%Y-%m-%d %H:%M:%S", self.converter(record.created))
        #prefix = '[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]' % \
        prefix = self.colored_format % \
            record.__dict__
        #if hasattr(self, "_color") and self._color:
        prefix = (self._colors.get(record.levelno, self._normal) +
                      prefix + self._normal)
        def safe_unicode(s):
            try:
                return _unicode(s)
            except UnicodeDecodeError:
                return repr(s)
        #message = (self._colors.get(record.levelno, self._normal) + record.message + self._normal)
        #formatted = prefix + " " + safe_unicode(message)
        formatted = prefix
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            # exc_text contains multiple lines.  We need to safe_unicode
            # each line separately so that non-utf8 bytes don't cause
            # all the newlines to turn into '\n'.
            lines = [formatted.rstrip()]
            lines.extend(safe_unicode(ln) for ln in record.exc_text.split('\n'))
            formatted = '\n'.join(lines)
        return formatted.replace("\n", "\n    ")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

import tornado
from tornado.httpserver import HTTPServer
from tornado.ioloop import IOLoop
from tornado.wsgi import WSGIContainer
from datetime import datetime
import logging
from code import interact
from tornado.options import define, options

define("cmd", default="runserver", metavar="runserver|urls")
define("port", default=8888)

tornado.options.parse_command_line()

def run(app, host='0.0.0.0', port=8888):
    if hasattr(app, "_wsgi") and app._wsgi == True:
        container = WSGIContainer(app); http = HTTPServer(container)
    else:
        http = HTTPServer(app, xheaders=True)
    #http.bind(port, host)
    port = options.port
    http.listen(port)
    #http.start()
    instance = IOLoop.instance()
    logging.info('- [%s is running on http://%s:%s]' % (app.__class__.__name__, host, port))
    if app.settings['debug'] == True and hasattr(app, "_wsgi") and app._wsgi == False:
        tornado.autoreload.start(instance)
        logging.info('- [%s restart with reload]' % (app.__class__.__name__))
    try:
        instance.start()
    except KeyboardInterrupt:
        instance.stop()

def show_urls(app):
    from torweb.urls import url_rules
    for urlspec in url_rules:
        #logging.info("%s %s %s" % (urlspec.regex.pattern, urlspec.handler_class, urlspec.kwargs))
        logging.info("%s <name:%s>" % (urlspec.regex.pattern, urlspec.name))





#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import soaplib
from lxml import etree
from soaplib.serializers import Base

_ns_xsi = soaplib.ns_xsi
_pref_soap_env = soaplib.prefmap[soaplib.ns_soap_env]

class Fault(Exception, Base):
    __type_name__ = "Fault"

    def __init__(self, faultcode='Server', faultstring="", detail=""):
        self.faultcode = '%s:%s' % (_pref_soap_env, faultcode)
        self.faultstring = faultstring
        self.detail = detail

    @classmethod
    def to_xml(cls, value, tns, parent_elt, name=None):
        if name is None:
            name = cls.get_type_name()
        element = etree.SubElement(parent_elt, "{%s}%s" % (soaplib.ns_soap_env,name))

        etree.SubElement(element, 'faultcode').text = value.faultcode
        etree.SubElement(element, 'faultstring').text = value.faultstring
        etree.SubElement(element, 'detail').text = value.detail

    @classmethod
    def from_xml(cls, element):
        code = element.find('faultcode').text
        string = element.find('faultstring').text
        detail_element = element.find('detail')
        if detail_element is not None:
            if len(detail_element.getchildren()):
                detail = etree.tostring(detail_element)
            else:
                detail = element.find('detail').text
        else:
            detail = ''
        return Fault(faultcode=code, faultstring=string, detail=detail)

    @classmethod
    def add_to_schema(cls, schema_dict):
        complex_type = etree.Element('complexType')
        complex_type.set('name', cls.get_type_name())
        sequenceNode = etree.SubElement(complex_type, 'sequence')

        element = etree.SubElement(sequenceNode, 'element')
        element.set('name', 'detail')
        element.set('{%s}type' % _ns_xsi, 'xs:string')

        element = etree.SubElement(sequenceNode, 'element')
        element.set('name', 'message')
        element.set('{%s}type' % _ns_xsi, 'xs:string')

        schema_dict.add_complex_type(cls, complex_type)

        top_level_element = etree.Element('element')
        top_level_element.set('name', 'ExceptionFaultType')
        top_level_element.set('{%s}type' % _ns_xsi, cls.get_type_name_ns())

        schema_dict.add_element(cls, top_level_element)
